@@ -46,6 +46,9 @@
 #' @param nl Logical; Indicates whether \code{formula} should be
 #'   treated as specifying a non-linear model. By default, \code{formula} 
 #'   is treated as an ordinary linear model formula.
+#' @param family Same argument as in \code{\link[brms:brm]{brm}}.
+#'   If \code{family} is specified \code{brmsformula}, it will overwrite
+#'   the value specified in \code{\link[brms:brm]{brm}}.
 #' @inheritParams brm
 #' 
 #' @return An object of class \code{brmsformula}, which
@@ -101,6 +104,13 @@
 #'   For more details on this model class see \code{\link[mgcv:gam]{gam}} 
 #'   and \code{\link[mgcv:gamm]{gamm}}.
 #'   
+#'   Gaussian process terms can be fitted using the \code{\link[brms:gp]{gp}}
+#'   function in the \code{pterms} part of the model formula. Similar to
+#'   smooth terms, Gaussian processes can be used to model complex non-linear
+#'   relationsships, for instance temporal or spatial autocorrelation. 
+#'   However, they are computationally demanding and are thus not recommended 
+#'   for very large datasets.
+#'   
 #'   The \code{pterms} and \code{gterms} parts may contain three non-standard
 #'   effect types namely monotonic, measurement error, and category specific effects,
 #'   which can be specified using terms of the form \code{mo(<predictors>)},
@@ -552,14 +562,6 @@ brmsformula <- function(formula, ..., flist = NULL, family = NULL,
     fix_theta <- lapply(fix_theta, "/", sum_theta)
     fix[names(fix_theta)] <- fix_theta
   }
-  if (!isTRUE(nl)) {
-    inv_names <- !is_auxpar_name(names(forms))
-    if (any(inv_names)) {
-      inv_names <- collapse_comma(names(forms)[inv_names])
-      stop2("The following parameter names are invalid: ", inv_names,
-            "\nSet 'nl = TRUE' if you want to specify non-linear models.")
-    }
-  }
   out$pforms[names(forms)] <- forms
   out$pfix[names(fix)] <- fix
   for (ap in names(out$pfix)) {
@@ -594,9 +596,28 @@ brmsformula <- function(formula, ..., flist = NULL, family = NULL,
   if (!is.null(family)) {
     out[["family"]] <- check_family(family)
   }
+  if (!is.null(out[["family"]])) {
+    # check for the presence of non-linear parameters
+    auxpars <- is_auxpar_name(names(out$pforms), out$family)
+    auxpars <- names(out$pforms)[auxpars]
+    for (ap in names(out$pforms)) {
+      if (!ap %in% auxpars) {
+        if (!isTRUE(out$nl)) {
+          stop2("The parameter '", ap, "' is not an auxiliary parameter.\n",
+                "Set 'nl = TRUE' if you want to specify non-linear models.")
+        }
+        # indicate the correspondence to an auxiliary parameter 
+        if (is.null(attr(out$pforms[[ap]], "auxpar"))) {
+          attr(out$pforms[[ap]], "auxpar") <- "mu"
+        }
+      }
+    }
+  }
   # add default values for unspecified elements
-  defs <- list(pforms = list(), pfix = list(), family = NULL, 
-               nl = FALSE, response = NULL, old_mv = FALSE)
+  defs <- list(
+    pforms = list(), pfix = list(), family = NULL, 
+    nl = FALSE, response = NULL, old_mv = FALSE
+  )
   defs <- defs[setdiff(names(defs), names(rmNULL(out, FALSE)))]
   out[names(defs)] <- defs
   class(out) <- "brmsformula"
@@ -769,8 +790,8 @@ pfix <- function(x, ...) {
   bf(x, ...)[["pfix"]]
 }
 
-amend_formula <- function(formula, data = NULL, family = NULL,
-                          nonlinear = NULL, partial = NULL) {
+amend_formula <- function(formula, data = NULL, family = gaussian(),
+                          nonlinear = NULL) {
   # incorporate additional arguments into formula
   # Args:
   #   formula: object of class 'formula' of 'brmsformula'
@@ -779,25 +800,15 @@ amend_formula <- function(formula, data = NULL, family = NULL,
   #   nonlinear, partial: deprecated arguments of brm
   # Returns:
   #   a brmsformula object compatible with the current version of brms
-  out <- bf(formula, family = family, nonlinear = nonlinear)
-  fnew <- ". ~ ."
-  if (!is.null(partial)) {
-    warning2("Argument 'partial' is deprecated. Please use the 'cs' ", 
-             "function inside the model formula instead.")
-    partial <- formula2str(partial, rm = 1)
-    fnew <- paste(fnew, "+ cs(", partial, ")")
+  out <- bf(formula, nonlinear = nonlinear)
+  if (is.null(out$family)) {
+    out <- bf(out, family = family)
   }
   # to allow the '.' symbol in formula
   try_terms <- try(terms(out$formula, data = data), silent = TRUE)
   if (!is(try_terms, "try-error")) {
     out$formula <- formula(try_terms)
   }
-  if (fnew != ". ~ .") {
-    out$formula <- update.formula(out$formula, formula(fnew))
-  }
-  if (is.null(out$family)) {
-    out$family <- check_family(gaussian())
-  }
   if (is_ordinal(out$family)) {
     # fix discrimination to 1 by default
     if (!"disc" %in% c(names(pforms(out)), names(pfix(out)))) {
