@@ -24,7 +24,7 @@
 #' names(data2)
 #'          
 #' @export
-make_standata <- function(formula, data, family = NULL, 
+make_standata <- function(formula, data, family = gaussian(), 
                           prior = NULL, autocor = NULL, nonlinear = NULL, 
                           cov_ranef = NULL, sample_prior = FALSE, 
                           knots = NULL, control = list(), ...) {
@@ -54,10 +54,11 @@ make_standata <- function(formula, data, family = NULL,
   bterms <- parse_bf(formula, family = family, autocor = autocor)
   check_prior_content(prior, family = family, warn = FALSE)
   na_action <- if (is_newdata) na.pass else na.omit
-  data <- update_data(data, family = family, bterms = bterms,
-                      drop.unused.levels = !is_newdata, 
-                      na.action = na_action, knots = knots,
-                      terms_attr = control$terms_attr)
+  data <- update_data(
+    data, bterms = bterms, na.action = na_action, 
+    drop.unused.levels = !is_newdata, knots = knots,
+    terms_attr = control$terms_attr
+  )
   
   # sort data in case of autocorrelation models
   if (has_arma(autocor) || is(autocor, "cor_bsts")) {
@@ -170,9 +171,8 @@ make_standata <- function(formula, data, family = NULL,
     resp <- bterms$response
     if (length(resp) > 1L && !old_mv) {
       args_eff_spec <- list(
-        x = bterms$auxpars[["mu"]],
-        smooth = control$smooth[["mu"]],
-        Jmo = control$Jmo[["mu"]]
+        x = bterms$auxpars[["mu"]], smooths = control$smooths[["mu"]],
+        gps = control$gps[["mu"]], Jmo = control$Jmo[["mu"]]
       )
       bterms$auxpars[["mu"]] <- NULL
       for (r in resp) {
@@ -190,9 +190,9 @@ make_standata <- function(formula, data, family = NULL,
     # data for predictors of auxiliary parameters
     for (ap in names(bterms$auxpars)) {
       args_eff_spec <- list(
-        x = bterms$auxpars[[ap]], nlpar = ap,
-        smooth = control$smooth[[ap]], 
-        Jmo = control$Jmo[[ap]]
+        x = bterms$auxpars[[ap]], nlpar = ap, 
+        smooths = control$smooths[[ap]],
+        gps = control$gps[[ap]], Jmo = control$Jmo[[ap]]
       )
       data_aux_eff <- do.call(data_effects, c(args_eff_spec, args_eff))
       standata <- c(standata, data_aux_eff)
