@@ -4,6 +4,12 @@ test_that("all S3 methods have reasonable ouputs", {
   fit3 <- brms:::rename_pars(brms:::brmsfit_example3)
   fit4 <- brms:::rename_pars(brms:::brmsfit_example4)
   fit5 <- brms:::rename_pars(brms:::brmsfit_example5)
+  fit6 <- brms:::rename_pars(brms:::brmsfit_example6)
+  
+  expect_range <- function(object, lower = -Inf, upper = Inf, ...) {
+    testthat::expect_true(all(object >= lower & object <= upper), ...)
+  }
+  SM <- suppressMessages
   
   # test S3 methods in alphabetical order
   # as.data.frame
@@ -28,12 +34,21 @@ test_that("all S3 methods have reasonable ouputs", {
                c(fit1$fit@sim$iter, length(parnames(fit1))))
   
   # coef
-  coef1 <- coef(fit1)
+  coef1 <- SM(coef(fit1))
+  expect_equal(dim(coef1$visit), c(4, 4, 8))
+  coef1 <- SM(coef(fit1, summary = FALSE))
+  expect_equal(dim(coef1$visit), c(nsamples(fit1), 4, 8))
+  coef2 <- SM(coef(fit2))
+  expect_equal(dim(coef2$patient), c(59, 4, 4))
+  coef4 <- SM(coef(fit4))
+  expect_equal(dim(coef4$subject), c(10, 4, 8))
+  # deprecated as of brms 1.7.0
+  coef1 <- coef(fit1, old = TRUE)
   expect_equal(dim(coef1$visit), c(4, 8))
-  coef2 <- coef(fit2)
+  coef2 <- coef(fit2, old = TRUE)
   expect_equal(dim(coef2[[2]]), c(59, 2))
   expect_equal(attr(coef2[[1]], "nlpar"), "a")
-  coef4 <- coef(fit4)
+  coef4 <- coef(fit4, old = TRUE)
   expect_equal(dim(coef4$subject), c(10, 8))
   
   # family
@@ -96,38 +111,51 @@ test_that("all S3 methods have reasonable ouputs", {
   fi <- fitted(fit5)
   expect_equal(dim(fi), c(nobs(fit5), 4))
   
+  fi <- fitted(fit6)
+  expect_equal(dim(fi), c(nobs(fit6), 4))
+  
   # fixef
-  fixef1 <- fixef(fit1, estimate = c("mean", "sd"))  
-  expect_equal(dimnames(fixef1), 
-    list(
-      c("Intercept", "sigma_Intercept", "Trt", "Age", 
-        "Trt:Age", "sAge_1", "sigma_Trt", "Exp"),
-      c("mean", "sd")
-    )
+  fixef1 <- SM(fixef(fit1))
+  expect_equal(rownames(fixef1), 
+    c("Intercept", "sigma_Intercept", "Trt", "Age", 
+      "Trt:Age", "sAge_1", "sigma_Trt", "Exp")
   )
+  # deprecated as of brms 1.7.0
+  fixef1 <- fixef(fit1, old = TRUE, estimate = c("mean", "sd"))  
+  expect_equal(rownames(fixef1), 
+   c("Intercept", "sigma_Intercept", "Trt", "Age", 
+     "Trt:Age", "sAge_1", "sigma_Trt", "Exp")
+  )
+  expect_equal(colnames(fixef1), c("mean", "sd"))
   fixef2 <- fixef(fit2, estimate = c("median", "quantile"), 
-                  probs = c(0.05, 0.95))
-  expect_equal(dimnames(fixef2),
-               list(c("a_Intercept", "a_Age", "b_Intercept", "b_Age"),
-                    c("median", "5%", "95%")))
+                  probs = c(0.05, 0.95), old = TRUE)
+  expect_equal(rownames(fixef2), 
+    c("a_Intercept", "a_Age", "b_Intercept", "b_Age")
+  )
+  expect_equal(colnames(fixef2), c("median", "5%", "95%"))
   
   # formula
   expect_equal(formula(fit1)$formula, 
     count ~ Trt * Age + mono(Exp) + s(Age) + offset(Age) + (1 + Trt | visit))
   
   # hypothesis
-  h1 <- hypothesis(fit1, "Intercept > Trt")
-  expect_equal(dim(h1$hypothesis), c(1, 6))
-  expect_output(print(h1), "(Intercept)-(Trt) > 0", fixed = TRUE)
-  expect_silent(p <- plot(h1, plot = FALSE))
+  hyp <- hypothesis(fit1, c("Intercept > Trt", "Trt:Age = -1"))
+  expect_equal(dim(hyp$hypothesis), c(2, 6))
+  expect_output(print(hyp), "(Intercept)-(Trt) > 0", fixed = TRUE)
+  expect_silent(p <- plot(hyp, plot = FALSE))
+  
+  hyp <- hypothesis(fit1, "Intercept = 0", class = "sd", group = "visit")
+  expect_true(is.numeric(hyp$hypothesis$Evid.Ratio[1]))
+  expect_output(print(hyp), "class sd_visit:", fixed = TRUE)
+  expect_silent(p <- plot(hyp, ignore_prior = TRUE, plot = FALSE))
+  
+  hyp <- hypothesis(fit1, "0 > r_visit[4,Intercept]", class = "", alpha = 0.01)
+  expect_equal(dim(hyp$hypothesis), c(1, 6))
+  expect_output(print(hyp, chars = NULL), "r_visit[4,Intercept]", fixed = TRUE)
+  expect_output(print(hyp), "l-99% CI", fixed = TRUE)
   
-  h2 <- hypothesis(fit1, "Intercept = 0", class = "sd", group = "visit")
-  expect_true(is.numeric(h2$hypothesis$Evid.Ratio[1]))
-  expect_output(print(h2), "class sd_visit:", fixed = TRUE)
-  expect_silent(p <- plot(h2, ignore_prior = TRUE, plot = FALSE))
   expect_error(hypothesis(fit1, "Intercept > x"), fixed = TRUE,
                "cannot be found in the model: \n'b_x'")
-  
   expect_error(hypothesis(fit1, 1),
                "Argument 'hypothesis' must be a character vector")
   expect_error(hypothesis(fit2, "b_Age = 0", alpha = 2),
@@ -136,9 +164,13 @@ test_that("all S3 methods have reasonable ouputs", {
   expect_error(hypothesis(fit3, "b_Age x 0"),
                "Every hypothesis must be of the form 'left (= OR < OR >) right'",
                fixed = TRUE)
-  expect_error(hypothesis(fit4, "x1 = 0", class = "a"),
-               "'a' is not a valid parameter class")
-
+  
+  # test hypothesis.default method
+  hyp <- hypothesis(as.data.frame(fit3), "bme_meAgeAgeSD > sigma")
+  expect_equal(dim(hyp$hypothesis), c(1, 6))
+  hyp <- hypothesis(fit3$fit, "bme_meAgeAgeSD > sigma")
+  expect_equal(dim(hyp$hypothesis), c(1, 6))
+  
   # omit launch_shiny
   
   # log_lik
@@ -175,6 +207,13 @@ test_that("all S3 methods have reasonable ouputs", {
   
   loo5 <- SW(LOO(fit5, cores = 1))
   expect_true(is.numeric(loo5[["looic"]]))
+  
+  loo6_1 <- SW(LOO(fit6, cores = 1))
+  expect_true(is.numeric(loo6_1[["looic"]]))
+  loo6_2 <- SW(LOO(fit6, cores = 1, newdata = fit6$data[1:30, ]))
+  expect_true(is.numeric(loo6_2[["looic"]]))
+  loo_compare <- compare_ic(loo6_1, loo6_2)
+  expect_range(loo_compare$ic_diffs__[1, 1], -1, 1)
 
   # loo_linpred
   llp <- SW(loo_linpred(fit1))
@@ -220,6 +259,15 @@ test_that("all S3 methods have reasonable ouputs", {
   meplot <- plot(me, stype = "raster", plot = FALSE)
   expect_true(is(meplot[[1]], "ggplot"))
   
+  me <- marginal_effects(fit1, "Trt", spaghetti = TRUE, nsamples = 10)
+  expect_equal(nrow(attr(me$Trt, "spaghetti")), 1000)
+  meplot <- plot(me, plot = FALSE)
+  expect_true(is(meplot[[1]], "ggplot"))
+  expect_error(
+    marginal_effects(fit1, "Trt", spaghetti = TRUE, surface = TRUE),
+    "Cannot use 'spaghetti' and 'surface' at the same time"
+  )
+  
   mdata = data.frame(
     Age = c(-0.3, 0, 0.3), 
     count = c(10, 20, 30), 
@@ -262,10 +310,17 @@ test_that("all S3 methods have reasonable ouputs", {
   me5 <- marginal_effects(fit5)
   expect_true(is(me5, "brmsMarginalEffects"))
   
+  me6 <- marginal_effects(fit6, nsamples = 100)
+  expect_true(is(me6, "brmsMarginalEffects"))
+  
   # marginal_smooths
-  ms1 <- marginal_smooths(fit1)
-  expect_equal(nrow(ms1[[1]]), 100)
-  expect_true(is(ms1, "brmsMarginalEffects"))
+  ms <- marginal_smooths(fit1)
+  expect_equal(nrow(ms[[1]]), 100)
+  expect_true(is(ms, "brmsMarginalEffects"))
+  
+  ms <- marginal_smooths(fit1, spaghetti = TRUE, nsamples = 10)
+  expect_equal(nrow(attr(ms[[1]], "spaghetti")), 1000)
+  
   expect_error(marginal_smooths(fit1, smooths = "s3"),
                "No valid smooth terms found in the model")
   expect_error(marginal_smooths(fit2),
@@ -296,6 +351,7 @@ test_that("all S3 methods have reasonable ouputs", {
                  "cor_patient__a_Intercept__b_Intercept", 
                  "r_patient__a[1,Intercept]", "r_patient__b[4,Intercept]",
                  "prior_b_a"))
+  expect_true(all(c("sdgp_gpAge", "lscale_gpAge") %in% parnames(fit6)))
   
   # plot tested in tests.plots.R
   
@@ -307,6 +363,10 @@ test_that("all S3 methods have reasonable ouputs", {
                c("b_Intercept", "b_sigma_Intercept", "b_Trt", 
                  "b_Age", "b_Trt:Age", "b_sAge_1", "b_sigma_Trt"))
   
+  # test default method
+  ps <- posterior_samples(fit1$fit, "b_Intercept")
+  expect_equal(dim(ps), c(nsamples(fit1), 1))
+  
   # posterior_predict
   expect_equal(dim(posterior_predict(fit1)), 
                c(nsamples(fit1), nobs(fit1)))
@@ -322,18 +382,15 @@ test_that("all S3 methods have reasonable ouputs", {
                  group = "visit", newdata = fit1$data[1:100, ])
   expect_true(is(pp, "ggplot"))
   
-  # uncomment as soon as bayesplot 1.2.0 is on CRAN
-  # pp <- SW(pp_check(fit1, type = "loo_pit", loo_args = list(cores = 1)))
-  # expect_true(is(pp, "ggplot"))
-  # lw <- SW(loo::psislw(-log_lik(fit1), cores = 1)$lw_smooth)
+  pp <- SW(pp_check(fit1, type = "loo_pit", loo_args = list(cores = 1)))
+  expect_true(is(pp, "ggplot"))
+  lw <- SW(loo::psislw(-log_lik(fit1), cores = 1)$lw_smooth)
   # not getting warnings implies that the precomputed lw is used
-  # pp <- pp_check(fit1, type = "loo_intervals", lw = lw)
-  # expect_true(is(pp, "ggplot"))
+  pp <- pp_check(fit1, type = "loo_intervals", lw = lw)
+  expect_true(is(pp, "ggplot"))
   
   expect_true(is(pp_check(fit3), "ggplot"))
   expect_true(is(pp_check(fit2, "ribbon", x = "Trt"), "ggplot"))
-  expect_error(pp_check(fit2, "ribbon"),
-               "Argument 'x' is required")
   expect_error(pp_check(fit2, "ribbon", x = "x"),
                "Variable 'x' is not a valid variable")
   expect_error(pp_check(fit1, "wrong_type"))
@@ -408,15 +465,26 @@ test_that("all S3 methods have reasonable ouputs", {
   expect_equal(dimnames(prs2), list(as.character(1:nsamples(fit1)), "b_Trt"))
   expect_equal(sort(prs1$b), sort(prs2$b_Trt))
   
+  # test default method
+  prs <- prior_samples(fit1$fit, pars = "^sd_visit")
+  expect_equal(names(prs), "prior_sd_visit")
+  
   # prior_summary
   expect_true(is(prior_summary(fit1), "brmsprior"))
   
   # ranef
-  ranef1 <- ranef(fit1, estimate = "median", var = TRUE)
+  ranef1 <- SM(ranef(fit1))
+  expect_equal(dim(ranef1$visit), c(4, 4, 2))
+  
+  ranef2 <- SM(ranef(fit2, summary = FALSE))
+  expect_equal(dim(ranef2$patient), c(nsamples(fit2), 59, 2))
+  
+  # deprecated as of brms 1.7.0
+  ranef1 <- ranef(fit1, estimate = "median", var = TRUE, old = TRUE)
   expect_equal(dim(ranef1$visit), c(4, 2))
   expect_equal(dim(attr(ranef1$visit, "var")), c(2, 2, 4))
   
-  ranef2 <- ranef(fit2, estimate = "mean", var = TRUE)
+  ranef2 <- ranef(fit2, estimate = "mean", var = TRUE, old = TRUE)
   expect_equal(dim(ranef2[[1]]), c(59, 1))
   expect_equal(dim(attr(ranef2[[1]], "var")), c(1, 1, 59))
   
@@ -465,7 +533,7 @@ test_that("all S3 methods have reasonable ouputs", {
                  "u-95% CI", "Eff.Sample", "Rhat"))
   expect_equal(rownames(summary1$random$visit), 
                c("sd(Intercept)", "sd(Trt)", "cor(Intercept,Trt)"))
-  expect_true(is.numeric(summary1$WAIC))
+  expect_true(is.numeric(summary1$waic))
   expect_output(print(summary1), "Population-Level Effects:")
   expect_output(print(summary1), "Priors:")
   
@@ -519,17 +587,28 @@ test_that("all S3 methods have reasonable ouputs", {
   expect_true(is(up, "brmsfit"))
   
   # VarCorr
-  vc <- VarCorr(fit1)
+  vc <- SM(VarCorr(fit1))
   expect_equal(names(vc), c("visit"))
   Names <- c("Intercept", "Trt")
-  expect_equivalent(dimnames(vc$visit$cov$mean), 
-                    list(Names, Names))
+  expect_equal(dimnames(vc$visit$cov)[c(1, 3)], list(Names, Names))
+  vc <- SM(VarCorr(fit2))
+  expect_equal(names(vc), c("patient"))
+  expect_equal(dim(vc$patient$cor), c(2, 4, 2))
+  vc <- SM(VarCorr(fit2, summary = FALSE))
+  expect_equal(dim(vc$patient$cor), c(nsamples(fit2), 2, 2))
+  
+  # deprecated as of brms 1.7.0
+  vc <- VarCorr(fit1, old = TRUE)
+  expect_equal(names(vc), c("visit"))
+  Names <- c("Intercept", "Trt")
+  expect_equivalent(dimnames(vc$visit$cov$mean), list(Names, Names))
   expect_output(print(vc), "visit")
   data_vc <- as.data.frame(vc)
   expect_equal(dim(data_vc), c(2, 7))
-  expect_equal(names(data_vc), c("Estimate", "Group", "Name", 
-                                 "Std.Dev", "Cor", "Cov", "Cov"))
-  vc <- VarCorr(fit2)
+  expect_equal(names(data_vc), 
+    c("Estimate", "Group", "Name", "Std.Dev", "Cor", "Cov", "Cov")
+  )
+  vc <- VarCorr(fit2, old = TRUE)
   expect_equal(names(vc), c("patient"))
   data_vc <- as.data.frame(vc)
   expect_equal(dim(data_vc), c(2, 7))
@@ -544,6 +623,9 @@ test_that("all S3 methods have reasonable ouputs", {
   expect_true(is.numeric(waic1[["se_waic"]]))
   expect_equal(waic1, SW(waic(fit1)))
   
+  fit1 <- SW(add_ic(fit1, "waic"))
+  expect_equal(WAIC(fit1), fit1$waic)
+  
   waic_compare <- SW(WAIC(fit1, fit1))
   expect_equal(length(waic_compare), 3)
   expect_equal(dim(waic_compare$ic_diffs__), c(1, 2))
